;; mgp-mode.el 
;;     --- MagicPoint mode 1.10
;;  Tested on Mule 2.3, based on GNU Emacs 19.28

;; Copyright (C) 1999  Hirotsugu Kakugawa

;; Maintainer: Hirotsugu Kakugawa (h.kakugawa@computer.org)
;;   1 Apr 1998 1.00  First Implementation
;;   2 Apr 1998 1.01  Added mgp-direc-emph. Changed key bindings.
;;   2 Apr 1998 1.10  Added jumping to error line in MagicPoint file
;;

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.


;; In your ~/.emacs
;; 
;; (setq auto-mode-alist
;;       (cons '("\\.mgp?\\'" . mgp-mode) 
;; 	    auto-mode-alist))
;; (autoload 'mgp-mode "mgp-mode")
;; (setq mgp-options "-g 800x600")


(require 'compile)

;; Program name of MagicPoint 
(defvar  mgp-program "mgp"
  "MagicPoint program name")

;; Command line option for MagicPoint
(defvar  mgp-options nil
  "MagicPoint optional arguments. nil or  string.")

;; History of mgp commands.
(defvar mgp-history nil)

;; Height of MagicPoint message window
(defvar mgp-window-height nil
  "*Number of lines in a MagicPoint window.  If nil, use Emacs default.")

(defvar  page-separator "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%"
  "Page separator string")
(defvar mgp-default-emph-thick-font "thick"
  "The font name for emphasized words, for mgp-direc-emph function.")
(defvar mgp-default-emph-thick-color "red"
  "The text color name for emphasized words, for mgp-direc-emph function.")
(defvar mgp-default-emph-normal-font "normal"
  "The font name for non-emphasized words, for mgp-direc-emph function.")
(defvar mgp-default-emph-normal-color "black"
  "The text color name for non-emphasized words, for mgp-direc-emph function.")

;; Syntax Table
(defvar mgp-mode-syntax-table nil
  "Syntax table used while in mgp mode.")

;; Abbrev Table
(defvar mgp-mode-abbrev-table nil
  "Abbrev table used while in mgp mode.")
(define-abbrev-table 'mgp-mode-abbrev-table ())

;; Syntax Table
(if mgp-mode-syntax-table
    ()
  (setq mgp-mode-syntax-table (make-syntax-table))
  (modify-syntax-entry ?\" ".   " mgp-mode-syntax-table)
  (modify-syntax-entry ?\\ ".   " mgp-mode-syntax-table)
  (modify-syntax-entry ?' "w   " mgp-mode-syntax-table))

;; Mode Map
(defvar mgp-mode-map nil
  "Keymap for Mgp (MagicPoint) mode.
Many other modes, such as Mail mode, Outline mode and Indented Mgp mode,
inherit all the commands defined in this map.")

(if mgp-mode-map
    ()
  (setq mgp-mode-map (make-sparse-keymap))
  (define-key mgp-mode-map "\e\t" 'ispell-complete-word)
  (define-key mgp-mode-map "\t" 'tab-to-tab-stop)
  (define-key mgp-mode-map "\C-c\C-v" 'mgp-run-mgp)
  (define-key mgp-mode-map "\C-c\C-c" 'kill-compilation)
  (define-key mgp-mode-map "\C-x`"    'next-error)
  (define-key mgp-mode-map "\C-c\C-f" 'mgp-direc-fore)
  (define-key mgp-mode-map "\C-c\C-b" 'mgp-direc-back)
  (define-key mgp-mode-map "\C-c\C-p" 'mgp-direc-page)
  (define-key mgp-mode-map "\C-c\C-l" 'mgp-direc-lcutin)
  (define-key mgp-mode-map "\C-c\C-r" 'mgp-direc-rcutin)
  (define-key mgp-mode-map "\C-c\C-i" 'mgp-direc-image)
  (define-key mgp-mode-map "\C-c\C-e" 'mgp-direc-emph)
  (define-key mgp-mode-map "\C-cc" 'mgp-direc-center-line)
  (define-key mgp-mode-map "\C-cl" 'mgp-direc-leftfill)
  (define-key mgp-mode-map "\C-cL" 'mgp-direc-left-line)
  (define-key mgp-mode-map "\C-cr" 'mgp-direc-right-line)
  (define-key mgp-mode-map "\C-cf" 'mgp-direc-font)
  (define-key mgp-mode-map "\C-cb" 'mgp-direc-bimage)
  (define-key mgp-mode-map "\C-ce" 'mgp-direc-embed)
  (define-key mgp-mode-map "\C-ci" 'mgp-direc-icon)
  (define-key mgp-mode-map "\C-cp" 'mgp-direc-pause) 
  (define-key mgp-mode-map "\C-ca" 'mgp-direc-again) 
  (define-key mgp-mode-map "\C-cm" 'mgp-direc-mark) 
  (define-key mgp-mode-map "\C-cn" 'mgp-direc-nodefault)
  (define-key mgp-mode-map "\C-cs" 'mgp-direc-size)
  (define-key mgp-mode-map "\C-cB" 'mgp-direc-bar)
  (define-key mgp-mode-map "\C-cC" 'mgp-direc-cont)
  (define-key mgp-mode-map "\C-cG" 'mgp-direc-bgrad)
  (define-key mgp-mode-map "\C-cP" 'mgp-direc-prefix) 
  (define-key mgp-mode-map "\C-cS" 'mgp-direc-system)
  (define-key mgp-mode-map "\C-cF" 'mgp-direc-filter))

;; MagicPoint mode
(defun mgp-mode ()
  "Major mode for editing MagicPoint files.

Special commands:
\\{mgp-mode-map}
Turning on Mgp mode calls the value of the variable `mgp-mode-hook',
if that value is non-nil."
  (interactive)
  (kill-all-local-variables)
  (use-local-map mgp-mode-map)
  (setq mode-name "MagicPoint")
  (setq major-mode 'mgp-mode)
  (setq local-abbrev-table mgp-mode-abbrev-table)
  (set-syntax-table mgp-mode-syntax-table)
  (make-local-variable 'compilation-window-height)
  (setq compilation-window-height mgp-window-height)
  (make-local-variable 'mgp-emph-thick-font)
  (setq mgp-emph-thick-font   mgp-default-emph-thick-font)
  (make-local-variable 'mgp-emph-thick-color)
  (setq mgp-emph-thick-color  mgp-default-emph-thick-color)
  (make-local-variable 'mgp-emph-normal-font)
  (setq mgp-emph-normal-font  mgp-default-emph-normal-font)
  (make-local-variable 'mgp-emph-normal-color)
  (setq mgp-emph-normal-color mgp-default-emph-normal-color)
  (run-hooks 'mgp-mode-hook))

;; Commands...

(defun mgp-direc-center-line ()
  (interactive)
  (mgp-insert-directive "center"))
(defun mgp-direc-left-line ()
  (interactive)
  (mgp-insert-directive "left"))
(defun mgp-direc-leftfill ()
  (interactive)
  (mgp-insert-directive "leftfill"))
(defun mgp-direc-right-line ()
  (interactive)
  (mgp-insert-directive "right"))
(defun mgp-direc-image ()
  (interactive)
  (mgp-insert-directive "image" t))
(defun mgp-direc-fore ()
  (interactive)
  (mgp-insert-directive "fore" t))
(defun mgp-direc-back ()
  (interactive)
  (mgp-insert-directive "back" t))
(defun mgp-direc-lcutin ()
  (interactive)
  (mgp-insert-directive "lcutin"))
(defun mgp-direc-rcutin ()
  (interactive)
  (mgp-insert-directive "rcutin"))
(defun mgp-direc-cont ()
  (interactive)
  (mgp-insert-directive "cont"))
(defun mgp-direc-nodefault ()
  (interactive)
  (mgp-insert-directive "nodefault"))
(defun mgp-direc-bar ()
  (interactive)
  (mgp-insert-directive "bar"))
(defun mgp-direc-pause ()
  (interactive)
  (mgp-insert-directive "pause"))
(defun mgp-direc-font ()
  (interactive)
  (mgp-insert-directive "font" t))
(defun mgp-direc-noop ()
  (interactive)
  (mgp-insert-directive "noop"))
(defun mgp-direc-system ()
  (interactive)
  (mgp-insert-directive "system" t))
(defun mgp-direc-bgrad ()
  (interactive)
  (mgp-insert-directive "bgrad"))
(defun mgp-direc-size ()
  (interactive)
  (mgp-insert-directive "size"))
(defun mgp-direc-prefix ()
  (interactive)
  (mgp-insert-directive "prefix" t))
(defun mgp-direc-icon ()
  (interactive)
  (mgp-insert-directive "icon"))
(defun mgp-direc-bimage ()
  (interactive)
  (mgp-insert-directive "bimage" t))
(defun mgp-direc-mark ()
  (interactive)
  (mgp-insert-directive "mark"))
(defun mgp-direc-again ()
  (interactive)
  (mgp-insert-directive "again"))

(defun mgp-insert-directive (dirc &optional arg)
  (beginning-of-line)
  (if (looking-at "%")
      (progn
	(end-of-line)
	(insert ", "))
    (insert "%"))
  (insert dirc)
  (if arg
      (if (stringp arg)
	  (progn 
	    (insert " ")
	    (insert arg))
	(insert " \"\"")
	(backward-char 1))))

(defun mgp-direc-page ()
  (interactive)
  (insert page-separator)
  (insert "\n%page\n"))

(defun mgp-direc-filter ()
  (interactive)
  (mgp-insert-directive2 "filter"))
(defun mgp-direc-embed ()
  (interactive)
  (mgp-insert-directive2 "embed"))
(defun mgp-insert-directive2 (direc)
  (insert (concat "%" direc "\n%end" direc "\n")))

(defun mgp-direc-emph ()
  (interactive)
  (insert "\n%cont")
  (if mgp-emph-thick-font
      (insert (concat ", font \"" mgp-emph-thick-font "\"")))
  (if mgp-emph-thick-color
      (insert (concat ", fore \"" mgp-emph-thick-color "\"")))
  (insert "\n%cont")
  (if mgp-emph-normal-font
      (insert (concat ", font \"" mgp-emph-normal-font "\"")))
  (if mgp-emph-normal-color
      (insert (concat ", fore \"" mgp-emph-normal-color "\"")))
  (insert "\n"))


(defun mgp-run-mgp (command-args)
  (interactive
   (list (read-from-minibuffer 
	  "Run MagicPoint: "
	  (concat mgp-program " " mgp-options)
	  nil nil 'mgp-history)))
  (if (buffer-modified-p)
      (if (y-or-n-p "MagicPoint document is modified. Save it? ")
	  (save-buffer)))
  (if (buffer-file-name (current-buffer))
      (compile-internal 
       (concat command-args " " 
	       (file-name-nondirectory (buffer-file-name (current-buffer))))
       "No more errors" "MagicPoint")))

;;; mgp-mode.el ends here
